/**************---< TECHNO ROAD.inc >---**************/
/*                                                   */
/*   File        :coron_uart.c                       */
/*   Version     :v3.0.0                             */
/*   Date        :2014/05/09                         */
/*   Author      :                                   */
/*   Description :Coron UART Library Source          */
/*                                                   */
/*    Copyright(C) TECHNO ROAD Inc.                  */
/*                 http://techno-road.com/           */
/*****************************************************/
#include "coron_conf.h"
#ifdef _CORON_UART
/* Includes ------------------------------------------------------------------*/
#include "stm32f10x_lib.h"
#include "coron_lib.h"

vu16 wp = 0;			// Cg|C^
vu16 rp = 0;			// [h|C^
char ring_buf[RING_BUFFER_SIZE];	// Oobt@

//==============================================================================
// CoronVA|[g֐
//==============================================================================
void UART_init(void){
	GPIO_InitTypeDef GPIO_InitStructure;
	USART_InitTypeDef USART_InitStructure;

	//CN-UART(XBee)|[g񓯊VAʐM|[gƂĐݒ
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_CN_UART, ENABLE);
	/* Configure USARTx_Tx as alternate function push-pull */
    GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = CN_UART_TX;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	/* Configure USARTx_Rx as input floating */
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = CN_UART_RX;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	//USART̐ݒ
	USART_StructInit(&USART_InitStructure);
	USART_InitStructure.USART_BaudRate = UART_BAUDRATE;			//{[[g
	USART_InitStructure.USART_WordLength = UART_LENGTH;			//f[^
	USART_InitStructure.USART_StopBits = UART_STOP;				//Xgbvrbg
	USART_InitStructure.USART_Parity = UART_PARITY;				//peB
	USART_InitStructure.USART_HardwareFlowControl = UART_Flow;	//t[
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	USART_Init(CN_UART, &USART_InitStructure);
	/* Enable the USARTx */
	USART_Cmd(CN_UART, ENABLE);

#ifdef USE_RING_BUFFER
	//荞ݐݒ
	NVIC_InitTypeDef NVIC_InitStructure;
   	NVIC_InitStructure.NVIC_IRQChannel= USART2_IRQn;
   	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 3;
   	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
   	NVIC_InitStructure.NVIC_IRQChannelCmd=ENABLE;
   	NVIC_Init(&NVIC_InitStructure);
   	//M荞݋
   	USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
   	//Oobt@
   	UART_init_ring_buf();
#endif
}

//==============================================================================
// UART Po
//==============================================================================
void UART_putc(char data){
	while( USART_GetFlagStatus(CN_UART, USART_FLAG_TXE ) == RESET );
	USART_SendData(CN_UART,data);
	while( USART_GetFlagStatus(USART2, USART_FLAG_TC ) == RESET );
}

//==============================================================================
// UART o
//==============================================================================
void UART_puts(char* buf){
	u16 cnt = 0;
	while(buf[cnt]!='\0'){
		UART_putc(buf[cnt]);
		cnt++;
	}
}

//==============================================================================
// UART w萔o(A)
//==============================================================================
void UART_putd(char* buf,u16 len){
	u32 cnt = 0;
	while(len){
		UART_putc(buf[cnt]);
		cnt++;
		len--;
	}
}

//==============================================================================
// UART P(0:M҂ 1:M҂L)
//==============================================================================
signed char UART_getc(char wf){
#ifndef USE_RING_BUFFER
	if(wf){
		// M܂őҋ@
		while( USART_GetFlagStatus(CN_UART, USART_FLAG_RXNE ) == RESET );
	}
	else{
		if( USART_GetFlagStatus(CN_UART, USART_FLAG_RXNE ) == RESET ){
			return -1;
		}
	}
	return USART_ReceiveData(CN_UART); // CMD or ACK
#else
	if(wf){
		while(wp == rp);
	}
	else{
		if(wp == rp){
			return -1;
		}
	}
	return UART_get_ring_buf();
#endif
}

//==============================================================================
// UART 
//==============================================================================
u16 UART_gets(char* buf){
	u16 cnt = 0;
	char c;
	while(1){
		c = (UART_getc(1));
		if(c == '\0'){
			buf[cnt]='\0';
		    break;
		}
	    else{
	    	buf[cnt] = c;
	    	cnt++;
	    }
	}
	return cnt;
}

//==============================================================================
// UART w萔(A)
//==============================================================================
u16 UART_getd(char* buf,u16 len){
	u32 cnt;
	signed char c = 0;
	for(cnt = 0; cnt < len; cnt++){
		c = UART_getc(1);
		if(c == -1){
			return (cnt + 1);
		}
		buf[cnt] = c;
	}
	return 0;
}

//==============================================================================
// UART oCi10iASCII16iϊ
//==============================================================================
char UART_put_x16(char num){
	if(num<10){
		return ('0'+num);
	}
	else{
		return ('A'+(num-10));
	}
}

//==============================================================================
// UART 16io(lA\)
//==============================================================================
void UART_putx(u32 num,char len){
	u8 s[12],n=2;
	s[0]='0';
	s[1]='x';
	switch(len){
		case  8:s[n]=UART_put_x16( num/0x10000000);n++;
		case  7:s[n]=UART_put_x16((num%0x10000000)/0x1000000);n++;
		case  6:s[n]=UART_put_x16((num%0x1000000)/0x100000);n++;
		case  5:s[n]=UART_put_x16((num%0x100000)/0x10000);n++;
		case  4:s[n]=UART_put_x16((num%0x10000)/0x1000);n++;
		case  3:s[n]=UART_put_x16((num%0x1000)/0x100);n++;
		case  2:s[n]=UART_put_x16((num%0x100)/0x10);n++;
		case  1:s[n]=UART_put_x16((num%0x10));n++;
	}
	UART_putd((char*)s,n);
}

//==============================================================================
// UART 10io(lA\)
//==============================================================================
void UART_putn(s32 num,char len){
	u8 s[10],n=1;
	if(num<0){s[0]='-';num=-num;}
	else{s[0]=' ';}
	switch(len){
		case  9:s[n]=(char)('0'+((num%1000000000)/100000000));n++;
		case  8:s[n]=(char)('0'+((num%100000000)/10000000));n++;
		case  7:s[n]=(char)('0'+((num%10000000)/1000000));n++;
		case  6:s[n]=(char)('0'+((num%1000000)/100000));n++;
		case  5:s[n]=(char)('0'+((num%100000)/10000));n++;
		case  4:s[n]=(char)('0'+((num%10000)/1000));n++;
		case  3:s[n]=(char)('0'+((num%1000)/100));n++;
		case  2:s[n]=(char)('0'+((num%100)/10));n++;
		case  1:s[n]=(char)('0'+((num%10)/1));n++;
	}
	UART_putd((char*)s,n);
}

//==============================================================================
// Oobt@
//==============================================================================
void UART_init_ring_buf(void){
	u16 i;
	for(i=0;i<RING_BUFFER_SIZE;i++){
		ring_buf[i] = 0;
	}
	wp = 0;
	rp = 0;
}

//==============================================================================
// Oobt@f[^擾
//==============================================================================
u16 UART_get_cap(void){
	if(wp >= rp){
		return (wp - rp);
	}
	return (u16)(((s32)U16_MAX + 1) + ((s32)wp - (s32)rp));
}

//==============================================================================
// Oobt@ǂݏo
//==============================================================================
char UART_get_ring_buf(void){
	return ring_buf[rp++ % RING_BUFFER_SIZE];
}

//==============================================================================
// Oobt@֏
//==============================================================================
void UART_put_ring_buf(char c){
	ring_buf[wp++ % RING_BUFFER_SIZE] = c;
}

#endif
